@@ -1,15 +1,17 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
-from copy import deepcopy
+import types
+import copy
 
 from django import forms
+from django.core.validators import EMPTY_VALUES
 from django.db import models
 from django.db.models.fields import FieldDoesNotExist
-from django.db.models.related import RelatedObject
 from django.utils import six
-from django.utils.datastructures import SortedDict
 from django.utils.text import capfirst
+from django.utils.translation import ugettext as _
+from sys import version_info
 
 try:
     from django.db.models.constants import LOOKUP_SEP
@@ -17,6 +19,19 @@ except ImportError:  # pragma: nocover
     # Django < 1.5 fallback
     from django.db.models.sql.constants import LOOKUP_SEP  # noqa
 
+try:
+    from collections import OrderedDict
+except ImportError:  # pragma: nocover
+    # Django < 1.5 fallback
+    from django.utils.datastructures import SortedDict as OrderedDict  # noqa
+
+try:
+    from django.db.models.related import RelatedObject as ForeignObjectRel
+except ImportError:  # pragma: nocover
+    # Django >= 1.8 replaces RelatedObject with ForeignObjectRel
+    from django.db.models.fields.related import ForeignObjectRel
+
+
 from .filters import (Filter, CharFilter, BooleanFilter,
     ChoiceFilter, DateFilter, DateTimeFilter, TimeFilter, ModelChoiceFilter,
     ModelMultipleChoiceFilter, NumberFilter)
@@ -25,6 +40,14 @@ from .filters import (Filter, CharFilter, BooleanFilter,
 ORDER_BY_FIELD = 'o'
 
 
+# There is a bug with deepcopy in 2.6, patch if we are running python < 2.7
+# http://bugs.python.org/issue1515
+if version_info < (2, 7, 0):
+    def _deepcopy_method(x, memo):
+        return type(x)(x.im_func, copy.deepcopy(x.im_self, memo), x.im_class)
+    copy._deepcopy_dispatch[types.MethodType] = _deepcopy_method
+
+
 def get_declared_filters(bases, attrs, with_base_filters=True):
     filters = []
     for filter_name, obj in list(attrs.items()):
@@ -44,7 +67,7 @@ def get_declared_filters(bases, attrs, with_base_filters=True):
             if hasattr(base, 'declared_filters'):
                 filters = list(base.declared_filters.items()) + filters
 
-    return SortedDict(filters)
+    return OrderedDict(filters)
 
 
 def get_model_field(model, f):
@@ -55,7 +78,7 @@ def get_model_field(model, f):
             rel = opts.get_field_by_name(name)[0]
         except FieldDoesNotExist:
             return None
-        if isinstance(rel, RelatedObject):
+        if isinstance(rel, ForeignObjectRel):
             model = rel.model
             opts = rel.opts
         else:
@@ -70,23 +93,42 @@ def get_model_field(model, f):
 
 def filters_for_model(model, fields=None, exclude=None, filter_for_field=None,
                       filter_for_reverse_field=None):
-    field_dict = SortedDict()
+    field_dict = OrderedDict()
     opts = model._meta
     if fields is None:
-        fields = [f.name for f in sorted(opts.fields + opts.many_to_many)]
+        fields = [f.name for f in sorted(opts.fields + opts.many_to_many)
+            if not isinstance(f, models.AutoField)]
+    # Loop through the list of fields.
     for f in fields:
+        # Skip the field if excluded.
         if exclude is not None and f in exclude:
             continue
         field = get_model_field(model, f)
+        # Do nothing if the field doesn't exist.
         if field is None:
             field_dict[f] = None
             continue
-        if isinstance(field, RelatedObject):
+        if isinstance(field, ForeignObjectRel):
             filter_ = filter_for_reverse_field(field, f)
+            if filter_:
+                field_dict[f] = filter_
+        # If fields is a dictionary, it must contain lists.
+        elif isinstance(fields, dict):
+            # Create a filter for each lookup type.
+            for lookup_type in fields[f]:
+                filter_ = filter_for_field(field, f, lookup_type)
+
+                if filter_:
+                    filter_name = f
+                    # Don't add "exact" to filter names
+                    if lookup_type != 'exact':
+                        filter_name = f + LOOKUP_SEP + lookup_type
+                    field_dict[filter_name] = filter_
+        # If fields is a list, it contains strings.
         else:
             filter_ = filter_for_field(field, f)
-        if filter_:
-            field_dict[f] = filter_
+            if filter_:
+                field_dict[f] = filter_
     return field_dict
 
 
@@ -135,6 +177,9 @@ class FilterSetMetaclass(type):
 
 
 FILTER_FOR_DBFIELD_DEFAULTS = {
+    models.AutoField: {
+        'filter_class': NumberFilter
+    },
     models.CharField: {
         'filter_class': CharFilter
     },
@@ -221,20 +266,27 @@ FILTER_FOR_DBFIELD_DEFAULTS = {
 class BaseFilterSet(object):
     filter_overrides = {}
     order_by_field = ORDER_BY_FIELD
+    strict = True
 
-    def __init__(self, data=None, queryset=None, prefix=None):
+    def __init__(self, data=None, queryset=None, prefix=None, strict=None):
         self.is_bound = data is not None
         self.data = data or {}
         if queryset is None:
             queryset = self._meta.model._default_manager.all()
         self.queryset = queryset
         self.form_prefix = prefix
+        if strict is not None:
+            self.strict = strict
 
-        self.filters = deepcopy(self.base_filters)
+        self.filters = copy.deepcopy(self.base_filters)
         # propagate the model being used through the filters
         for filter_ in self.filters.values():
             filter_.model = self._meta.model
 
+        # Apply the parent to the filters, this will allow the filters to access the filterset
+        for filter_key, filter_ in six.iteritems(self.filters):
+            filter_.parent = self
+
     def __iter__(self):
         for obj in self.qs:
             yield obj
@@ -248,28 +300,51 @@ class BaseFilterSet(object):
     @property
     def qs(self):
         if not hasattr(self, '_qs'):
+            valid = self.is_bound and self.form.is_valid()
+
+            if self.strict and self.is_bound and not valid:
+                self._qs = self.queryset.none()
+                return self._qs
+
+            # start with all the results and filter from there
             qs = self.queryset.all()
             for name, filter_ in six.iteritems(self.filters):
-                try:
-                    if self.is_bound:
-                        data = self.form[name].data
-                    else:
-                        data = self.form.initial.get(
-                            name, self.form[name].field.initial)
-                    val = self.form.fields[name].clean(data)
-                    qs = filter_.filter(qs, val)
-                except forms.ValidationError:
-                    pass
+                value = None
+                if valid:
+                    value = self.form.cleaned_data[name]
+                else:
+                    raw_value = self.form[name].value()
+                    try:
+                        value = self.form.fields[name].clean(raw_value)
+                    except forms.ValidationError:
+                        # for invalid values either:
+                        # strictly "apply" filter yielding no results and get outta here
+                        if self.strict:
+                            self._qs = self.queryset.none()
+                            return self._qs
+                        else:  # or ignore this filter altogether
+                            pass
+
+                if value is not None:  # valid & clean data
+                    qs = filter_.filter(qs, value)
+
             if self._meta.order_by:
+                order_field = self.form.fields[self.order_by_field]
+                data = self.form[self.order_by_field].data
+                ordered_value = None
                 try:
-                    order_field = self.form.fields[self.order_by_field]
-                    data = self.form[self.order_by_field].data
-                    value = order_field.clean(data)
-                    if value:
-                        qs = qs.order_by(value)
+                    ordered_value = order_field.clean(data)
                 except forms.ValidationError:
                     pass
+
+                if ordered_value in EMPTY_VALUES and self.strict:
+                    ordered_value = self.form.fields[self.order_by_field].choices[0][0]
+
+                if ordered_value:
+                    qs = qs.order_by(*self.get_order_by(ordered_value))
+
             self._qs = qs
+
         return self._qs
 
     def count(self):
@@ -278,7 +353,7 @@ class BaseFilterSet(object):
     @property
     def form(self):
         if not hasattr(self, '_form'):
-            fields = SortedDict([
+            fields = OrderedDict([
                 (name, filter_.field)
                 for name, filter_ in six.iteritems(self.filters)])
             fields[self.order_by_field] = self.ordering_field
@@ -297,12 +372,18 @@ class BaseFilterSet(object):
                     # e.g. (('field', 'Display name'), ...)
                     choices = [(f[0], f[1]) for f in self._meta.order_by]
                 else:
-                    choices = [(f, capfirst(f)) for f in self._meta.order_by]
+                    choices = [(f, _('%s (descending)' % capfirst(f[1:])) if f[0] == '-' else capfirst(f))
+                               for f in self._meta.order_by]
             else:
+                # add asc and desc field names
                 # use the filter's label if provided
-                choices = [(fltr.name or f, fltr.label or capfirst(f))
-                           for f, fltr in self.filters.items()]
-            return forms.ChoiceField(label="Ordering", required=False,
+                choices = []
+                for f, fltr in self.filters.items():
+                    choices.extend([
+                        (fltr.name or f, fltr.label or capfirst(f)),
+                        ("-%s" % (fltr.name or f), _('%s (descending)' % (fltr.label or capfirst(f))))
+                    ])
+            return forms.ChoiceField(label=_("Ordering"), required=False,
                                      choices=choices)
 
     @property
@@ -311,14 +392,18 @@ class BaseFilterSet(object):
             self._ordering_field = self.get_ordering_field()
         return self._ordering_field
 
+    def get_order_by(self, order_choice):
+        return [order_choice]
+
     @classmethod
-    def filter_for_field(cls, f, name):
+    def filter_for_field(cls, f, name, lookup_type='exact'):
         filter_for_field = dict(FILTER_FOR_DBFIELD_DEFAULTS)
         filter_for_field.update(cls.filter_overrides)
 
         default = {
             'name': name,
-            'label': capfirst(f.verbose_name)
+            'label': capfirst(f.verbose_name),
+            'lookup_type': lookup_type
         }
 
         if f.choices:
@@ -346,7 +431,7 @@ class BaseFilterSet(object):
     @classmethod
     def filter_for_reverse_field(cls, f, name):
         rel = f.field.rel
-        queryset = f.model._default_manager.all()
+        queryset = f.field.model._default_manager.all()
         default = {
             'name': name,
             'label': capfirst(rel.related_name),
